//+------------------------------------------------------------------+
//|                                               ^L_Correlation.mq4 |
//|                                        Copyright  2008, lotos4u |
//|                                                lotos4u@gmail.com |
//+------------------------------------------------------------------+
#property copyright "Copyright  2008, lotos4u"
#property link      "lotos4u@gmail.com"

//+------------------------------------------------------------------+
//| defines                                                          |
//+------------------------------------------------------------------+
#define CLOSE_MODE 0 //  
#define CLOSEOPEN_MODE 1 //      
#define CLOSE_RELATIVE_MODE 2 //      
#define CLOSEOPEN_RELATIVE_MODE 3 //         
#define FULL_AVG_MODE 4 //   (  4)
#define FULL_AVG_RELATIVE_MODE 5 //  ,   

#define HIGHLOW_MODE 6 //      
#define HIGHLOW_AVG_MODE 7 //    .  . 
#define HIGHLOW_RELATIVE_MODE 8 //   .  .    . 
#define HIGHLOW_AVG_RELATIVE_MODE 9 //    ( .  . )  .  


#property indicator_separate_window

#property indicator_minimum -1
#property indicator_maximum 1

#property indicator_buffers 3

#property indicator_color1 Aqua // 
#property indicator_color2 Blue //   . 
#property indicator_color3 Red //   

#property indicator_width1 1 //  . 
#property indicator_width2 2 //   
#property indicator_width3 1 //  

#property indicator_level1 0.75
#property indicator_level2 0.5
#property indicator_level3 0.0
#property indicator_level4 -0.5
#property indicator_level5 -0.75
#property indicator_levelcolor BlueViolet

#include <stderror.mqh>
#include <stdlib.mqh>

extern int      Mode = FULL_AVG_MODE;                     // ,   
extern bool     AutoCorrelation = false;                  // ,   Pair       AutoCorrelationShift
extern string   Pair = "GBPUSD";                          //  ,    

extern bool     ShowCorrelation = false;                   // ,     
extern bool     ShowMA = false;                           // ,     
extern bool     ShowHistogram = true;                     // ,      
extern bool     ShowSummary = true;                       // ,    

extern int      ShowEssentialPoints = -1;                  //    .  =0,    ,  =-1,   
extern int      ShowEssentialRanges = -1;                  //    ( )  .  =0,   ,  =-1,   
extern int      ShowEssentialRectangles = -1;              //   (  )  .  =0,    ,  =-1,   

extern int      CorrelationRadius = 0;                    // ,      
extern int      MA_Period = 10;                           //  
extern int      ResultingBars = 0;                        // ,        
extern int      AutoCorrelationShift = 0;                 //      ,      (   )
extern double   EssentialLevel = 0.7;                     // , ..   .      

extern string   FontName = "Verdana";                     // ,      
extern int      FontSize = 9;                            // 

extern color    FontColor = White;                        // 
extern color    CorrelationColor = indicator_color1;      //  . 
extern color    MAColor = indicator_color2;               //   
extern color    HistoColor = Orange;                      //   . 
extern color    EssentialPointsColor = indicator_color3;  //  
extern color    EssentialRangesColor = Red;               //   
extern color    EssentialRectanglesColor = Olive;         //   


double CorrelationBuffer[];
double AverageCorrelationBuffer[];
double EssentialPointsBuffer[];


double Correlation[], AvgCorrelation[], ArrayX[], ArrayY[], ValidX[], ValidY[];
double AverageX, AverageY, AverageXY, DispersionX, DispersionY, CovariationXY, HistogramMax = 0.0, AverageCorrelation;
int CalculateCounter, ValidDataCounter, AverageLength, ValidDataLength;
string ShortName = "L_Correlation", HistoName = " Histo ", SummaryLabel = " Label ";
int IndicatorWindow, HistogramBars = 200, HistogramValues[];





////////////////////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////////////////////
int init()
{
   //   ,        (   !)
   if(Pair == "")
      Pair = Symbol();

   //   
   if(CorrelationRadius == 0)
   {
      switch(Period())
      {
         case PERIOD_M1: CorrelationRadius = 10; break;//10 min
         case PERIOD_M5: CorrelationRadius = 12; break;//hour
         case PERIOD_M15: CorrelationRadius = 12; break;//3 hours
         case PERIOD_M30: CorrelationRadius = 12; break;//6 hours
         case PERIOD_H1: CorrelationRadius = 12; break;//12 hours
         case PERIOD_H4: CorrelationRadius = 6; break;//week
         case PERIOD_D1: CorrelationRadius = 5; break;//month
         case PERIOD_W1: CorrelationRadius = 12; break;//year
         case PERIOD_MN1: CorrelationRadius = 6; break;//year
      }
   }
   
   //   
   if(AutoCorrelation && AutoCorrelationShift == 0)
   {
      switch(Period())
      {
         case PERIOD_M1: AutoCorrelationShift = 60; break;//hour
         case PERIOD_M5: AutoCorrelationShift = 12; break;//hour
         case PERIOD_M15: AutoCorrelationShift = 96; break;//day
         case PERIOD_M30: AutoCorrelationShift = 48; break;//day
         case PERIOD_H1: AutoCorrelationShift = 120; break;//week
         case PERIOD_H4: AutoCorrelationShift = 30; break;//week
         case PERIOD_D1: AutoCorrelationShift = 22; break;//month
         case PERIOD_W1: AutoCorrelationShift = 52; break;//year
         case PERIOD_MN1: AutoCorrelationShift = 12; break;//year
      }
   }
      
   //         
   if(AutoCorrelation)
      ShortName = ShortName + " (" + Symbol() + " Auto, " + Mode + ", " + CorrelationRadius + ", " + AutoCorrelationShift + ", " + MA_Period + ")";
   else
      ShortName = ShortName + " (" + Symbol() + "-" + Pair + ", " + Mode + ", " + CorrelationRadius + ", " + MA_Period + ")";
   
   IndicatorShortName(ShortName);
   
   //   
   SetIndexStyle(0, DRAW_LINE, EMPTY, EMPTY, CorrelationColor);
   SetIndexBuffer(0, CorrelationBuffer);
   SetIndexEmptyValue(0, 2.0);

   //     . 
   SetIndexStyle(1, DRAW_LINE, EMPTY, EMPTY, MAColor);
   SetIndexBuffer(1, AverageCorrelationBuffer);
   SetIndexEmptyValue(1, 2.0);
   
   //   
   SetIndexStyle(2, DRAW_ARROW, EMPTY, EMPTY, EssentialPointsColor);
   SetIndexArrow(2, 108);
   SetIndexBuffer(2, EssentialPointsBuffer);
   SetIndexEmptyValue(2, 2.0);
   
   CalculateCounter = 0;
   ValidDataCounter = 0;
   
   CreateSummaryLabels();
   
   return(0);
}
////////////////////////////////////////////////////////////////////////////////////////////




////////////////////////////////////////////////////////////////////////////////////////////
//      
////////////////////////////////////////////////////////////////////////////////////////////
int deinit()
{
   CalculateCounter = 0;
   ValidDataCounter = 0;
   DeleteObjectsByPhrase(ShortName);
   return(0);
}
////////////////////////////////////////////////////////////////////////////////////////////





////////////////////////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////////////////////////
int start()
{
   CreateSummaryLabels();
    //      
   if(CalculateCounter > 0 && Volume[0] > 1)
   {
      return(0);
   }
   int error, essCounter, limit;
   string Name;
   IndicatorWindow = WindowFind(ShortName);

   if(AutoCorrelation)
      ValidDataLength = Bars;
   else
      ValidDataLength = MathMin(Bars, iBars(Pair, 0));

   CreateValidData();
   
   ArrayResize(Correlation, ValidDataLength);ArrayInitialize(Correlation, 0.0);ArraySetAsSeries(Correlation, true);
   ArrayResize(AvgCorrelation, ValidDataLength);ArrayInitialize(AvgCorrelation, 0.0);ArraySetAsSeries(AvgCorrelation, true);
   
   for(int i = 0; i < ValidDataLength; i++)
   {
      CreateDataArrays(i);
      Correlation[i]  = getCorrelation(ArrayX, ArrayY);
      
      if(ShowCorrelation)
         CorrelationBuffer[i] = Correlation[i];
   }
   
   //     . 
   for(i = 0; i < ValidDataLength; i++)
   {
      AvgCorrelation[i] = iMAOnArray(Correlation, 0, MA_Period, 0, MODE_SMA, i);
      if(ShowMA)//   -   
         AverageCorrelationBuffer[i] = AvgCorrelation[i];
   }

   essCounter = 0;//  
   for(i = 0; i < ValidDataLength; i++)
   {
      if((MathAbs(Correlation[i]) > EssentialLevel) && ((MathAbs(Correlation[i]) > MathAbs(Correlation[i-1])) && (MathAbs(Correlation[i]) > MathAbs(Correlation[i+1]))))
      {
         essCounter++;//   
         
         //   
         if((ShowEssentialPoints == 0) || (essCounter <= ShowEssentialPoints))
         {
            EssentialPointsBuffer[i] = Correlation[i];
         }

         //  
         if(AutoCorrelation && (ShowEssentialRanges == 0) || (essCounter <= ShowEssentialRanges))
         {
            Name = ShortName + " VLine1 " + i;
            ObjectCreate(Name, OBJ_VLINE, 0, Time[i], Correlation[i]);
            ObjectSet(Name, OBJPROP_COLOR, EssentialRangesColor);

            Name = ShortName + " VLine2 " + i;
            ObjectCreate(Name, OBJ_VLINE, 0, Time[AutoCorrelationShift + i], Correlation[i]);
            ObjectSet(Name, OBJPROP_COLOR, EssentialRangesColor);
         }
         
         //  
         if(AutoCorrelation && (ShowEssentialRectangles == 0) || (essCounter <= ShowEssentialRectangles))
         {
            Name = ShortName + " Rect1 " + i;
            ObjectCreate(Name, OBJ_RECTANGLE, 0, Time[CorrelationRadius + i], High[iHighest(Symbol(), 0, MODE_HIGH, CorrelationRadius, i)], Time[i], Low[iLowest(Symbol(), 0, MODE_LOW, CorrelationRadius, i)]);
            ObjectSet(Name, OBJPROP_COLOR, EssentialRectanglesColor);
            ObjectSet(Name, OBJPROP_BACK, true);

            Name = ShortName + " Rect2 " + i;
            ObjectCreate(Name, OBJ_RECTANGLE, 0, Time[CorrelationRadius + AutoCorrelationShift + i], High[iHighest(Symbol(), 0, MODE_HIGH, CorrelationRadius, AutoCorrelationShift+i)], Time[AutoCorrelationShift+i], Low[iLowest(Symbol(), 0, MODE_LOW, CorrelationRadius, AutoCorrelationShift+i)]);
            ObjectSet(Name, OBJPROP_COLOR, EssentialRectanglesColor);
            ObjectSet(Name, OBJPROP_BACK, true);
         }
      }
   }
   
   //  ,     . 
   if(ResultingBars == 0)
      AverageLength = ValidDataCounter;
   else
      AverageLength = ResultingBars;
   
      
   //  
   AverageCorrelation = iMAOnArray(Correlation, 0, AverageLength, 0, MODE_SMA, 0);
   
   
   // ,        
   HistogramMax = 0.0;
   if(ValidDataLength > 0)
      CreateHistogramArray();
   
   
   //    
   if(ShowHistogram && ValidDataLength > 0)
   {
      //  
      DeleteObjectsByPhrase(ShortName + HistoName);
      double StartX, StartY, EndX, EndY;
      
      //   
      for(i = -HistogramBars/2; i < HistogramBars/2; i++)
      {
         if(HistogramValues[i + HistogramBars/2] == 0)continue;
         
         Name = ShortName + HistoName + i;//   
         StartX  = Time[0];//   
         StartY  = i*2.0/HistogramBars;//   
         EndX    = Time[0 + 1*HistogramValues[i + HistogramBars/2]];//   
         EndY    = StartY;//   
         
         ObjectCreate(Name, OBJ_TREND, IndicatorWindow, StartX, StartY, EndX, EndY);
         ObjectSet(Name, OBJPROP_RAY, false);
         ObjectSet(Name, OBJPROP_BACK, true);
         ObjectSet(Name, OBJPROP_COLOR, HistoColor);
      }
   }
   
   //  
   if(ShowSummary)
      ShowSummary(essCounter);
   
   CalculateCounter = MathMin(1, ValidDataLength);
   
   return(0);
}
////////////////////////////////////////////////////////////////////////////////////////////






////////////////////////////////////////////////////////////////////////////////////////////
// ,        
//(  )
////////////////////////////////////////////////////////////////////////////////////////////
void CreateHistogramArray()
{
   double level, levelNext, Upper = 1.0, Lower = -1.0, step;
   int size = ArraySize(Correlation), maxValue = 0;
   ArrayResize(HistogramValues, HistogramBars);
   ArrayInitialize(HistogramValues, 0);
   //   ""  
   //  HistogramBars     -1  +1
   //     ,    
   step = (Upper - Lower)/HistogramBars;//"" 
   for(int j = 0; j < ArraySize(Correlation); j++)
   {
      for(int i = 0; i < HistogramBars; i++)   
      {
         level = (i-HistogramBars/2)*step;//  
         levelNext = (i-HistogramBars/2+1)*step;//  
         if(Correlation[j] >= level && Correlation[j] < levelNext)//       
         {
            HistogramValues[i]++;//    1
            if(maxValue < HistogramValues[i])// ,     
            {
               maxValue = HistogramValues[i];
               HistogramMax = level;
            }
            break;//     
         }
      }
   }
}
////////////////////////////////////////////////////////////////////////////////////////////






////////////////////////////////////////////////////////////////////////////////////////////
//    
////////////////////////////////////////////////////////////////////////////////////////////
void ShowSummary(int essCounter)
{
   string Text1, Text2;
   if(AutoCorrelation)
      Text2 = "";
   else
      //      
      Text2 = ": . /MA = " + DoubleToStr(HistogramMax, 2) + "/" + DoubleToStr(AverageCorrelation, 2) + " (" + AverageLength + "  " + ValidDataLength + " )";
      
   //   
   Text1 = "  (>" + DoubleToStr(EssentialLevel, 2) + "): " + essCounter;
   ObjectSetText(ShortName + SummaryLabel + "1", Text1, FontSize, FontName, FontColor);
   ObjectSetText(ShortName + SummaryLabel + "2", Text2, FontSize, FontName, FontColor);
}
////////////////////////////////////////////////////////////////////////////////////////////



////////////////////////////////////////////////////////////////////////////////////////////
//    
////////////////////////////////////////////////////////////////////////////////////////////
double getX(int shift)
{
   return(getSymbol(Symbol(), shift, Mode));
}
////////////////////////////////////////////////////////////////////////////////////////////





////////////////////////////////////////////////////////////////////////////////////////////
//    
////////////////////////////////////////////////////////////////////////////////////////////
double getY(int shift)
{
   if(AutoCorrelation)
      return(getSymbol(Symbol(), AutoCorrelationShift + shift, Mode));
   else
      return(getSymbol(Pair, shift, Mode));
}
////////////////////////////////////////////////////////////////////////////////////////////





////////////////////////////////////////////////////////////////////////////////////////////
// ,      
////////////////////////////////////////////////////////////////////////////////////////////
bool isValidPriceX(int shift)
{
   return(iOpen(Symbol(), 0, shift) > 0.0);
}
////////////////////////////////////////////////////////////////////////////////////////////






////////////////////////////////////////////////////////////////////////////////////////////
// ,      
////////////////////////////////////////////////////////////////////////////////////////////
bool isValidPriceY(int shift)
{
   if(AutoCorrelation)
      return(iOpen(Symbol(), 0, AutoCorrelationShift + shift) > 0.0);
   else
      return(iOpen(Pair, 0, shift) > 0.0);
}
////////////////////////////////////////////////////////////////////////////////////////////






////////////////////////////////////////////////////////////////////////////////////////////
//        . 
////////////////////////////////////////////////////////////////////////////////////////////
void CreateDataArrays(int StartBar)
{
   ArrayResize(ArrayX, CorrelationRadius); ArrayResize(ArrayY, CorrelationRadius);
   ArrayInitialize(ArrayX, 0.0); ArrayInitialize(ArrayY, 0.0);
   datetime timeX, timeY;
   int DataCounter = 0, yIndex = StartBar, xIndex = StartBar;
   while(xIndex < StartBar + CorrelationRadius || xIndex < ArraySize(ValidX))
   {
      if(ValidX[xIndex] > 0.0 && ValidY[yIndex] > 0.0)//   
      {
         ArrayX[DataCounter] = ValidX[xIndex]; ArrayY[DataCounter] = ValidY[yIndex];
         DataCounter++;
      }
      yIndex++; xIndex++;      
   }
   //      -  ,    
   if(DataCounter < CorrelationRadius)
      {ArrayResize(ArrayX, 0); ArrayResize(ArrayY, 0);}
}
////////////////////////////////////////////////////////////////////////////////////////////






////////////////////////////////////////////////////////////////////////////////////////////
//   ,       
//   
////////////////////////////////////////////////////////////////////////////////////////////
void CreateValidData()
{
   ArrayResize(ValidX, ValidDataLength); ArrayResize(ValidY, ValidDataLength);
   ArrayInitialize(ValidX, 0.0); ArrayInitialize(ValidY, 0.0);
   datetime timeX, timeY;
   int yIndex = - 1;
   ValidDataCounter = 0;
   for(int xIndex = 0; xIndex < ValidDataLength; xIndex++)
   {
      yIndex++;
      timeX = iTime(Symbol(), 0, xIndex); timeY = iTime(Pair, 0, yIndex);
      if(timeX == NULL || timeY == NULL)break;
      if(AutoCorrelation)
      {
         if(!(isValidPriceX(xIndex) && isValidPriceY(yIndex)))
         {
            Sleep(2000);
            if(!(isValidPriceX(xIndex) && isValidPriceY(yIndex)))
               continue;
         }
         ValidX[xIndex] = getX(xIndex); ValidY[yIndex] = getY(yIndex);
         ValidDataCounter++;
      }
      else
      {
         if(timeX == timeY)
         {
            if(!(isValidPriceX(xIndex) && isValidPriceY(yIndex)))//   
            {
               Sleep(2000);// 2 
               if(!(isValidPriceX(xIndex) && isValidPriceY(yIndex)))//   
                  continue;//    -    
            }
            ValidX[xIndex] = getX(xIndex); ValidY[yIndex] = getY(yIndex);
            ValidDataCounter++;
         }
         else if(timeX > timeY)
         {
            xIndex++;//       
         }               
         else if(timeX < timeY)
         {
            yIndex++;//      
         }
      }
   }
}
////////////////////////////////////////////////////////////////////////////////////////////





////////////////////////////////////////////////////////////////////////////////////////////
//      
//(  ,     ,    )
////////////////////////////////////////////////////////////////////////////////////////////
double getCorrelation(double X[], double Y[])
{
   if(ArraySize(X) != ArraySize(Y) || ArraySize(X) == 0)return(0.0);
   double avgX = 0, avgY = 0, covarXY = 0, dispX = 0, dispY = 0;
   int i, length = ArraySize(X);
   
   for(i = 0; i < length; i++)
      {avgX += X[i];avgY += Y[i];}
   avgX /= length;avgY /= length;
   
   for(i = 0; i < length; i++)
   {
      covarXY += (X[i]-avgX)*(Y[i]-avgY);
      dispX += (X[i] - avgX)*(X[i] - avgX);
      dispY += (Y[i] - avgY)*(Y[i] - avgY);
   }
   covarXY /= length; dispX /= length; dispY /= length;
   
   if(MathSqrt(dispX*dispY) > 0.0)
      return(covarXY/MathSqrt(dispX*dispY));
   else
      return(0.0);
}
////////////////////////////////////////////////////////////////////////////////////////////





////////////////////////////////////////////////////////////////////////////////////////////
//       
////////////////////////////////////////////////////////////////////////////////////////////
void CreateSummaryLabels()
{
   IndicatorWindow = WindowFind(ShortName);
   if(IndicatorWindow == -1)return(-1);
   string Name = ShortName + SummaryLabel + "1";
   if(ObjectFind(Name) == -1 && ObjectCreate(Name, OBJ_LABEL, IndicatorWindow, 0, 0))
   {
      ObjectSet(Name, OBJPROP_CORNER, 2);
      ObjectSet(Name, OBJPROP_XDISTANCE, 0);
      ObjectSet(Name, OBJPROP_YDISTANCE, 0.7*FontSize);
   }
   Name = ShortName + SummaryLabel + "2";
   if(ObjectFind(Name) == -1 && ObjectCreate(Name, OBJ_LABEL, IndicatorWindow, 0, 0))
   {
      ObjectSet(Name, OBJPROP_CORNER, 2);
      ObjectSet(Name, OBJPROP_XDISTANCE, 0);
      ObjectSet(Name, OBJPROP_YDISTANCE, 2.5*FontSize);
   }
}
////////////////////////////////////////////////////////////////////////////////////////////




////////////////////////////////////////////////////////////////////////////////////////////
//  ,     
//Phrase (   )
////////////////////////////////////////////////////////////////////////////////////////////
void DeleteObjectsByPhrase(string Phrase)
{
   string ObjName;
   for(int i = ObjectsTotal()-1; i >= 0; i--)
   {
      ObjName = ObjectName(i);
      if(StringFind(ObjName, Phrase, 0) > -1) 
      { 
         ObjectDelete(ObjName);
      }
   }
}
////////////////////////////////////////////////////////////////////////////////////////////


double getSymbol(string Smbl, int shift, int Mode, int TimeFrame = 0)
{
   double
      h = iHigh(Smbl, TimeFrame, shift),
      o = iOpen(Smbl, TimeFrame, shift),
      c = iClose(Smbl, TimeFrame, shift),
      l = iLow(Smbl, TimeFrame, shift);
      
   switch(Mode)
   {
      case CLOSE_MODE: return(c);
      case CLOSEOPEN_MODE: return(c - o);
      case CLOSE_RELATIVE_MODE: return(c/h);
      case CLOSEOPEN_RELATIVE_MODE: return((c - o)/h);
      
      case HIGHLOW_MODE: return(h - l);
      case HIGHLOW_RELATIVE_MODE: return((h - l)/h);
      case HIGHLOW_AVG_MODE: return((h + l)/2.0);
      case HIGHLOW_AVG_RELATIVE_MODE: return((h + l)/(2.0*h));

      case FULL_AVG_MODE: return((h + l + c + o)/4.0);
      case FULL_AVG_RELATIVE_MODE: return((h + l + c + o)/(4.0*h));
   }
}

